@@ -356,9 +356,13 @@ handle_spec_block.specCAPTURE <- function(x, ...) {
   scrape_and_call(x, pass = "CAPTURE", narrow = TRUE, ...)
 }
 
+#' @param etas allows for block capture of ETAs in the simulated output;
+#' this should be R code that will get parsed and evaluated; the result should 
+#' be an integer-like vector which identifies which ETAs will be captured.
+#' 
 #' @rdname BLOCK_PARSE
-CAPTURE <- function(x, env, pos = 1, annotated = FALSE, ...) {
-  
+CAPTURE <- function(x, env, pos = 1, annotated = FALSE, 
+                    etas = NULL, ...) {
   if(annotated) {
     context <- env[["incoming_names"]][pos]
     context <- glue("parse annotated capture block ({context})")
@@ -372,7 +376,14 @@ CAPTURE <- function(x, env, pos = 1, annotated = FALSE, ...) {
     x <- cvec_cs(x)
   }
   
-  check_block_data(x, env, pos)
+  if(!is.null(etas)) {
+    if(is.logical(etas)) {
+      abort("`etas` must be text, not a logical value.")  
+    }
+    env[["capture_etas"]] <- c(env[["capture_etas"]], etas)
+  } else {
+    check_block_data(x, env, pos)
+  }
   
   env[["capture"]][[pos]] <- x
   
@@ -849,7 +860,7 @@ handle_spec_block.specYAML <- function(x, env, ...) {
     x <- lapply(x, FUN = do.call, what = what) 
     x <- bind_rows(x)
     label <-  names(data) 
-    if(is.null(labels) & is.character(data)) label <- data
+    if(is.null(label) & is.character(data)) label <- data
     mutate(x, block = block, name = label)
   }
   names(x) <- tolower(names(x))
