@@ -1,4 +1,4 @@
-# Copyright (C) 2013 - 2021 Metrum Research Group
+# Copyright (C) 2013 - 2023 Metrum Research Group
 #
 # This file is part of mrgsolve.
 #
@@ -159,7 +159,7 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
                   capture = NULL,
                   preclean = FALSE, recover = FALSE, ...) {
   
-  if(charthere(model, "/")) {
+  if(!identical(basename(model), as.character(model))) {
     project <- dirname(model)
     model <- basename(model)
   }
@@ -214,7 +214,7 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
   mread.env <- parse_env(
     spec,
     incoming_block_names,
-    project = build[["project"]], 
+    build,
     ENV
   )
   
@@ -272,9 +272,10 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
   
   # capture  ----
   capture_more <- capture
-  capture_code <- unlist(do.call("c", nonull.list(mread.env[["capture"]])))
-  capture <- .ren.create(as.character(capture_code))
-  annot <- capture_param(annot,.ren.new(capture))
+  capture <- unlist(nonull.list(mread.env[["capture"]]))
+  capture <- .ren.create(capture)
+  capture <- .ren.sanitize(capture)
+  annot <- capture_param(annot, .ren.new(capture))
   
   # Collect potential multiples
   subr  <- collect_subr(spec)
@@ -398,6 +399,11 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
   }
   
   # more captures ----
+  
+  # Process @etas 1:n -----
+  x <- capture_etas(x, mread.env)
+  
+  # Process capture passed into mread -----
   if(is.character(capture_more)) {
     valid_capture <- get_valid_capture(
       param = param, omega = omega, sigma = sigma, build = build, 
@@ -407,6 +413,7 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
       capture_more <- valid_capture[valid_capture != "."]  
     }
     capture_vars <- .ren.create(capture_more)
+    capture_vars <- .ren.sanitize(capture_vars)
     if(!all(capture_vars[["old"]] %in% valid_capture)) {
       bad <- setdiff(capture_vars[["old"]], valid_capture)
       for(b in bad) {
@@ -418,13 +425,10 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
         call. = FALSE
       )
     }
-    capture_code <- unique(c(capture_code, capture_more))
-    capture <- .ren.create(capture_code)
-    x@capture <- .ren.chr(capture)
-    x <- default_outputs(x)
+    x <- update_capture(x, .ren.chr(capture_vars))
     build$preclean <- TRUE
   }
-  
+
   # Check mod ----
   check_pkmodel(x, subr, spec)
   check_globals(mread.env[["move_global"]], Cmt(x))
@@ -476,7 +480,8 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
   }
   
   cat(
-    paste0("// Source MD5: ", build[["md5"]], "\n"),
+    paste0("// Source MD5: ", build[["md5"]]),
+    "\n// PLUGINS:",
     plugin_code(plugin),
     ## This should get moved to rd
     "\n// FIXED:",
@@ -488,9 +493,9 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
     "\n// MODEL HEADER FILES:",
     incl("mrgsolv.h"), 
     incl("modelheader.h"),
-    "\n//INCLUDE databox functions:",
+    "\n// INCLUDE databox functions:",
     incl("databox_cpp.h"),
-    "\n//USING plugins",
+    "\n// USING plugins:",
     plugin_using(plugin),
     "\n// GLOBAL CODE BLOCK:",
     "// GLOBAL VARS FROM BLOCKS & TYPEDEFS:",
@@ -531,7 +536,7 @@ mread <- function(model, project = getOption("mrgsolve.project", getwd()),
     dbs[["cmt"]],
     table,
     spec[["PRED"]],
-    write_capture(.ren.old(capture)),
+    write_capture(names(x@capture)),
     "__END_table__",
     "", 
     sep="\n", file=def.con)
@@ -599,6 +604,11 @@ mread_cache <- function(model = NULL,
                         preclean = FALSE, 
                         capture = NULL, ...) {
   
+  if (!identical(basename(model), as.character(model))) {
+    project <- dirname(model)
+    model <- basename(model)
+  }
+  
   if(is.character(capture)) preclean <- TRUE
   
   build <- new_build(file, model, project, soloc, code, preclean) 
