@@ -565,6 +565,86 @@ void dataobject::carry_out(const recstack& a,
   }
 }
 
+
+// Looks at `Data` and finds ETA1, ETA2, ETA3 etc and forms matrix of 
+// ETA to use in place of ETAS simulated from OMEGA.
+arma::mat dataobject::get_etas(const int n_eta, const bool strict, 
+                               const std::string& value) {
+  
+  Rcpp::CharacterVector::iterator bg = Data_names.begin();
+  Rcpp::CharacterVector::iterator ed = Data_names.end();
+  
+  std::vector<int> eta_location(n_eta);
+
+  int n_found = 0; 
+  int n_missing = 0;
+  unsigned int column_index;
+  unsigned int end_index = Data_names.size()-1;
+  
+  std::string prefix = "ETA";
+  
+  for(int i = 1; i <= n_eta; ++i) {
+    
+    prefix = i >= 10 ? "ET" : "ETA";
+    
+    if(i == 10) {
+      Rcpp::Function asNamespace("asNamespace");
+      Rcpp::Environment mrgsolve_ns = asNamespace("mrgsolve");
+      Rcpp::Function check_names = mrgsolve_ns["check_etasrc_names"];
+      check_names(Data_names, n_eta, value); 
+    }
+    
+    std::string eta_label = prefix + std::to_string(i);
+    column_index = std::find(bg, ed, eta_label) - bg;
+    
+    if(i >= 10 && column_index > end_index) {
+      eta_label = "ETA" + std::to_string(i);
+      column_index = std::find(bg, ed, eta_label) - bg;
+    }
+    
+    if(column_index <= end_index) {
+      eta_location[i-1] = column_index;
+      ++n_found;
+    } else {
+      eta_location[i-1] = -1;  
+      ++n_missing;
+    }
+  }
+  
+  if(strict && n_missing > 0) {
+    throw Rcpp::exception(
+        tfm::format(
+          "all %i ETAs must be provided when `etasrc` is \"%s\".", 
+          n_eta, value
+        ).c_str(),
+        false
+    );
+  }
+  
+  if(n_found==0) {
+    throw Rcpp::exception(
+        tfm::format(
+          "at least one ETA must be provided when `etasrc` is \"%s\".", 
+          value
+        ).c_str(),
+        false
+    );
+  }
+  
+  arma::mat ans(Uid.size(), n_eta); 
+  for(size_t i = 0; i < Uid.size(); ++i) {
+    int start_row = Startrow[i];
+    for(int j = 0; j < n_eta; ++j) {
+      if(eta_location[j] > 0) {
+        ans(i, j) = Data(start_row, eta_location[j]);
+      }
+    }
+  }
+  
+  return ans;
+}
+
+
 // SAVE FOR LATER
 // void dataobject::expand_records(recstack& a, 
 //                                 dataobject& idat,
